#Folder: structural_model

This folder estimates our structural model of the patient's accept/reject decision
and compute assignment under counterfactual mechanisms.

MAIN CODE FILES

The following main code files in structural_model need to be run in the following order:

	1. compileMex.sh
		- This script compiles the C code used for counterfactual analysis
	2. import_all.sh
		- This script converts primary_files (generated by gen_primary_files.do) to .mat format
	3. run_model.m
		- This script is a wrapper function for estimating structural model of the patient's 
		  accept/reject decision
		- This script runs four different specifications:
			i. nyrt_fix （base specification)
			ii. nyrt_fix_uh (donor unobserved heterogeneity)
			iii. nyrt_fix_time_uh (donor unobserved heterogeneity and time controls)
			iv. nyrt_fix_time_pt_don_uh (for robustness: patient unobserved heterogeneity)
		- Model files called by this script are saved in code/model_files
		- This script generates results for the following figures and tables:
			i. Table 5: Conditional Choice Probability of Acceptance (select coefficients)
			ii. Figure 1: Model Fit
			iii. Figure 2: Value of an Organ Offer
			iv. Table A.3: Conditional Choice Probability of Acceptance (Detailed)
	4. run_model_validation.m
		- This script is similar to run_model.m, except this file estimates structural model
		  for the validation period
		- This script needs to be run multiple times for a complete re-run. See the instruction
		  in the beginning of the code.
		- This script generates results for the following table:
			i. Table A.4: Out-of-sample Model Validation
	5. run_cf.m
		- This script completes the counterfactual analysis 
		- This script needs to be run multiple times for a complete re-run. See the instruction
		  in the beginning of the code.
		- This script runs the following counterfactual mechanisms:
			i. score_binspoints_locl_ref (Pre-2014 Priorities)
			ii. score_kas (Post-2014 Priorities)
			iii. score_fifo (First Come First Served)
			iv. score_lifo (Last Come First Served)
			v. optim_second (Optimal Offer Rates)
			vi. greedy_second (Approximately Optimal Priorities)
			vii. greedy_secondC (Approx. Opt. Pareto Improving Priorities)
			viii. optim_first (Optimal Assignment)
			ix. score_gammaeta (Greedy Priorities)
			x. score_binspoints_fact_locl (Pre-2014 Priorities [Naive])
			xi. score_kas_fact (Post-2014 Priorities [Naive])
			xii. score_fifo_fact (First Come First Served [Naive])
			xiii. score_lifo_fact (Last Come First Served [Naive])
			xiv. score_gammaeta_fact (Greedy Priorities [Naive])
			xv. optim_secondC (Opt. Pareto Improving Offer Rates)
		- This script generates results for the following table:
			i. Figure 3: Welfare and Organ Utilization under Alternative Mechanisms
			ii. Table 6: Outcomes in Various Mechanisms
			iii. Table 7: Sources of Welfare Gains
			iv. Table C.5: Outcomes under Alternative Modeling Assumptions



DIRECTORY STRUCTURE

The following main subfolders are in the /structural_model/code directory:
	1. model_files: the model files that define each model to be estimated are stored here
	2. cf_files, cf_functions, cf_optimal: scipts and functions used in generating 
	   counterfactual predictions
	3. functions: functions used in structural model of the patient's accept/reject decision
	4. gibbsdiag/, mcmcdiag/: functions to perform diagnostics on gibbs sampler output

The structural estimation generates a master data file for each model stored in 
~/organs-star/model_master_files/<model_name>/.




