% This is the zero mismatch model
model = struct();
priors = struct();

% Filepath
model.model_name = 'nyrt_fix_time_uh';
model.local_opo  = 12;
model.gen_script = mfilename('fullpath'); % name of script that generated model file (i.e. name of this script)
model.gen_date = datetime('today'); % date model file was generated

% Filepaths for input data
primary_data_folder = '~/organs-star/primary_files/nyrt/';
model.filenames.offers = strcat(primary_data_folder,'simulated_offers_matlab');
model.filenames.pt_history = strcat(primary_data_folder,'pt_history');
model.filenames.donors = strcat(primary_data_folder,'donors');
model.filenames.patients = strcat(primary_data_folder,'patients');
model.filenames.don_pat_pairs = strcat(primary_data_folder,'don_pat_pairs_');
model.filenames.hazard_folder = '~/organs-proj/common_output/reduced_form/hazard/';
model.pos_xmatch.folder = '/proj/organs/common_output/reduced_form/';
model.pos_xmatch.coefs = [model.pos_xmatch.folder 'positive_crossmatch_coefs_rich.csv'];

% Model structure
model.donor_uh = true;
model.pt_uh = false;

% Constants, and renormalize
model.renormalize = true;
model.q_leave = 'trim_gompertz';
model.rho = - log(0.95)/365;  % Discount factor
model.K_types = 1;          % Number of types
model.constant = true;      % Should a constant be included? %
                             % Donor creatinine categories are exhaustive
if model.renormalize, model.constant = true; end;

% Offers variables
model.n_offers.hazard = 'timeout_hazard';

% Sample
model.choice_def = {'accept_810_yn', 'accept'}; % Sample choice variable ('accept' means 1<->accept)
model.offer_def = 'any([offers.offered_dat==1 offers.screened==1],2)'; % Sample offer definition 
model.match_date_range = {'01/01/2010', '12/31/2013'}; % sample time period

% Offer Sorting
model.offer_sort = {'position_sim'};

% Covariates
model.donor_chars = {'age_don','don_age_lt_18','don_age_18_35', ...
                    'don_age_ge_50','don_cod_anoxia', ...
                    'don_cod_stroke','don_cod_cns', ...
                    'don_cod_other','don_creat_lt_p5', ...
                    'don_creat_p5_1p0','don_creat_1p0_1p5', ...
                    'don_creat_ge_1p5','don_pa_off_or_tx', ...
                    'ecd_donor','dcd','don_male', ...
                    'hist_hypertens_don','opo_dsa_id', 'opo_region_id'};
%    List of donor characteristics 'don_cod_other' is excluded because a constant has been included

model.patient_chars = {'init_date','init_age','pt_diabetes','abo', ...
                    'end_bmi_calc','tot_serum_album', ...
                    'num_prev_tx','dialysis_date'};    % List of patient characteristics
model.pt_history = {'cpra'}; 

% List of history-specific patient characteristics
model.offer_chars = {'match_date','geo','a_mm','b_mm','dr_mm','c_abo'};    % List of match-specific characteristics from offer file
model.time_var = {'init_date','match_date'}; % Start-time and match_time. Empty is not incl
model.newvars = {{{model.time_var{1},model.time_var{2}},@(x) (x{2}-x{1})/365,'wait_time'},...
                 {'wait_time', @(x) log(min(max(x,1/365),7)), 'log_wait_time'}, ...              
                 {'a_mm',@(x) x==2,'amis_2'},... 
                 {'b_mm',@(x) x==2,'bmis_2'},... 
                 {'dr_mm',@(x) x==2,'drmis_2'},... 
                 {{'a_mm','b_mm','dr_mm'},@(x) (x{1} + x{2} + x{3})==0,'zero_mm'},... 
                 {'c_abo',@(x) x~=2,'abo_compatible'},... % 2 = abo identical
                 {'abo',@(x) any([x==1 x==11 x==12],2),'pt_abo_A'},... 
                 {'abo',@(x) x==0,'pt_abo_O'},... 
                 {'opo_dsa_id',@(x) x==model.local_opo, 'nyrt_donor'},...
                 {{'opo_region_id','opo_dsa_id'},@(x) (x{1}==9).*(x{2}~=model.local_opo), 'regional_donor'},...
                 {'opo_region_id',@(x) x~=9, 'national_donor'},...
                 {{'geo','opo_dsa_id'},@(x) (x{1}==1).*(x{2}~=model.local_opo),'non_nyrt_local_match'},...
                 {'hist_hypertens_don',@(x) x==1,'donor_hypertension'}, ...
                 {'num_prev_tx',@(x) x>0,'prior_transplant'},...
                 {{'init_date','dialysis_date'},@(x) x{1} >= x{2},'on_dialysis_at_init_date'},...
                 {{'init_date','dialysis_date'},@(x) (x{2}>0).*log(min(max(x{1}-x{2},1),5000)/365),'log_dialysis_time_at_init'},... % Conditioning on non-missing dialysis
                 {'cpra',@(x) x>=0.8, 'cpra_gt80'},...; 
                 {'cpra',@(x) x==0,   'cpra_0'},...
		 {'cpra',@(x) (x-0.8).*(x>=0.8),'cpra_g_80'}}; 
% {var1,var2,...}}; % {var1,var2,...}


model.norm_scale = {}; % Scale variables
model.missing = {'yes', -99, 'yes'};  % Are there missing values coded with a number, whats the code, create dummies?

% Covariates with fixed coefficients across patient types
model.fixed.linear = {'pt_diabetes','cpra','cpra_gt80','cpra_0', ...
		      'cpra_g_80','prior_transplant', ...
                    'don_age_lt_18', 'don_age_18_35','don_age_ge_50', ...
                    'don_cod_anoxia','don_cod_stroke', 'don_cod_cns', ...
                    'don_creat_p5_1p0', 'don_creat_1p0_1p5', ...
                    'don_creat_ge_1p5', 'don_pa_off_or_tx', ...
                    'ecd_donor','dcd', 'don_male','donor_hypertension', ...
                    'zero_mm', 'amis_2', 'bmis_2', 'drmis_2', ...
                    'abo_compatible','regional_donor','national_donor', ...
                    'non_nyrt_local_match', 'pt_abo_A', 'pt_abo_O', ...
                    'on_dialysis_at_init_date'};,

model.fixed.polyvar = {}; % {poly1,poly2,...} A "poly" is a tuple {{vars},order}, so one "poly" per order
model.fixed.splines = {}; % {tensor1,tensor2,...}
% A "spline" is a tuple {var,knots,order,minval,maxval}. A "tensor" is a tuple {spline1,spline2,..}
% For example: {{{'age',5,3,0,90},{'age_don',5,3,0,90}},{{'log_wait_time',7,3,0,3650}}} Two set of splines. The first one 
% consists of the interactions between age and age_don. The second one is a spline on wait time with 7 knots and 
% degree 3 between 0 and 3650. 

% Bmi knots based on normal ranges
% Albumin knots based on 25th and 75th percentiles
model.fixed.pwlin = {{'init_age',[18,35,50,65]},{'log_wait_time',log([1 2 3])},...
		     {'end_bmi_calc',[18.5 25 30]},{'tot_serum_album',[3.7 4.4]},...
		     {'log_dialysis_time_at_init',log(5)}};

model.fixed.interactions ={...
                           {'zero_mm','prior_transplant'},...
                           {'zero_mm','pt_diabetes'},...
                           {'zero_mm','init_age'},...
                           {'zero_mm','cpra'},...
                           {'zero_mm','cpra_gt80'},...
                           {'zero_mm','ecd_donor'},...
                           {'zero_mm','dcd'},...
                           {'zero_mm','nyrt_donor'},...
                           {'zero_mm','abo_compatible'},...
                           {'nyrt_donor','amis_2'},...
                           {'nyrt_donor','bmis_2'},...
                           {'nyrt_donor','drmis_2'},...
                           {'nyrt_donor','don_age_lt_18'},...
                           {'nyrt_donor','don_age_18_35'},...
                           {'nyrt_donor','don_age_ge_50'},...
                           {'init_age','don_age_lt_18'},...
                           {'init_age','don_age_18_35'},...
                           {'init_age','don_age_ge_50'},...
                           {'init_age_gt_35','don_age_18_35'},...
                           {'init_age_gt_35','don_age_ge_50'},...
                           {'log_wait_time','prior_transplant'}, ...
                           {'log_wait_time','pt_diabetes'}, ...
                           {'log_wait_time','init_age'}, ...
                           {'log_wait_time','cpra'}, ...
                           {'log_wait_time','cpra_gt80'}, ...
                           {'log_wait_time','tot_serum_album'}, ...
                           {'log_wait_time','end_bmi_calc'}, ...
                           {'log_wait_time','pt_abo_A'}, ...
                           {'log_wait_time','pt_abo_O'}};  % B or AB is omitted

% Covariates with flexible coefficients across patient types
model.flex.linear = {};  
model.flex.polyvar = {}; 
model.flex.splines = {};
model.flex.pwlin = {};
model.flex.interactions = {};

%Typical donor set
model.typical_donor_set = @(donor_data) donor_data.opo_dsa_id==model.local_opo;
% Gamma and/or V projection
model.gamma = 'default';
model.v = 'default';

% Priors
priors.V_nu = 2;   % prior for unobserved heterogeneity
priors.V_theta = 100; % prior variance for CCP parameter vector (very dispersed)
priors.mu_theta = 0; % prior for mean of CCP parameter vector
priors.V_eta = 1; % Prior for the variance of donor UH

% store model in /structural_model/input/ (../input/)
save([models_folder model.model_name],'model','priors');

